/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jxva.cache.strong;

import java.util.LinkedHashMap;

import com.jxva.cache.Cache;
import com.jxva.cache.CacheException;
import com.jxva.cache.Element;

/**
 * Last In First Out
 * @author  The Jxva Framework Foundation
 * @since   1.0
 * @version 2009-01-23 09:58:38 by Jxva
 */
public class LIFOSoftCache extends AbstractStrongCache {

	private static final long serialVersionUID = 1L;
	
	public LIFOSoftCache(String name){
		this(name,DEFAULT_MAX_SIZE);
	}
	
	public LIFOSoftCache(String name,int maxSize){
		this.name=name;
		this.maxSize=maxSize;
		this.map=new LinkedHashMap<Object,Element>();
	}
	
	protected final void doPut(Element element) throws CacheException {
        if (isFull()) {
            removeLastElement();
        }
    }

    Element getLastElement() {
        if (map.size() == 0) {
            return null;
        }
        Object[] keys = map.keySet().toArray();
        return map.get(keys[keys.length-1]);
    }
    
    private void removeLastElement() throws CacheException {
        remove(getLastElement().getKey());
    }
    
	public static void main(String[] args) {
        Cache cache = new LIFOSoftCache("ddd",5);  
        cache.put(1, "1");  
        cache.put(2, "2");  
        cache.put(3, "3");  
        cache.put(4, "4");  
        cache.getValue(4);
        cache.put(5, "5");  
        cache.put(6, "6");  
        cache.put(7, "7");  
        cache.getValue(4);
        cache.put(8, "8");  
        cache.put(9, "9");  

        String value = (String)cache.getValue(4);  
        System.out.println(value);  
        System.out.println("Testing ...");  
        
        Object[] keys= cache.getKeys(); 
        System.out.println(keys.length);
        for (Object key:keys) {  
            System.out.println(cache.get(key));  
        } 		
	}

}
