/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jxva.cache.weak;

import java.lang.ref.WeakReference;
import java.util.concurrent.Callable;
import java.util.concurrent.CompletionService;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;

import com.jxva.cache.Cache;
import com.jxva.cache.Element;

/**
 *
 * @author  The Jxva Framework Foundation
 * @since   1.0
 * @version 2009-01-23 10:23:05 by Jxva
 */
public class FastWeakCache extends AbstractWeakCache {

	private static final long serialVersionUID = 1L;
	

	public FastWeakCache(String name){
		this(name,DEFAULT_MAX_SIZE);
	}
	
	public FastWeakCache(String name,int maxSize){
		this.name=name;
		this.maxSize=maxSize;
		this.map=new ConcurrentHashMap<Object,WeakReference<Element>>();
	}
	
	private static final long s=System.nanoTime();
	private static final AtomicInteger count=new AtomicInteger(0);
	private static final int MAX_THREAD=1000;
	
	public static void main(String[] args) {
		 Cache cache = new FastWeakCache("ddd",5);  
	        cache.put(1, "1");  
	        cache.put(2, "2");  
	        cache.put(3, "3");  
	        cache.put(4, "4");  
	        cache.getValue(4);
	        cache.put(5, "5");  
	        cache.put(6, "6");  
	        cache.put(7, "7");  
	        cache.getValue(4);
	        cache.put(8, "8");  
	        cache.put(9, "9");  

	        String value = (String)cache.getValue(4);  
	        System.out.println(value);  
	        System.out.println("Testing ...");  
	        
	        Object[] keys= cache.getKeys(); 
	        System.out.println(keys.length);
	        for (Object key:keys) {  
	            System.out.println(cache.get(key));  
	        }		
	}
	
	public static void multiThread(){
		ExecutorService exec = Executors.newCachedThreadPool();
		CompletionService<Integer> completionService =new ExecutorCompletionService<Integer>(exec);

		for (int index = 0; index < MAX_THREAD; index++) {
			final int NO=index;
			completionService.submit(new Callable<Integer>() {
				public Integer call() {
					try{
						FastWeakCache cache = new FastWeakCache("test");
						cache.put("test",NO);
						Thread.sleep(1000L);
						if(NO!=(Integer)cache.getValue("test")){
							System.out.println(false);
						}
					}catch(Exception e){
						e.printStackTrace();
					}
					count.getAndIncrement();
					return count.intValue();
				}
			});
		};
		try {
			for(int i=0;i<MAX_THREAD;i++){
				if(MAX_THREAD==completionService.take().get()){
					System.out.println((System.nanoTime()-s)+" ns");
				}
			}
			System.out.println("---end---");
		} catch (Exception e) {
			e.printStackTrace();
		}
		exec.shutdown();
	}
}
