/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jxva.dao;

import com.jxva.util.Assert;

/**
 * 
 * @author The Jxva Framework Foundation
 * @since 1.0
 * @version 2009-01-03 21:09:38 by Jxva
 */
public class PageBean {
	
	public static final int DEFAULT_PAGESIZE = 20;
	private int pagesize=20;
	private int pageno;
	private long totalPage;
	private long totalCount;
	
	public PageBean(){
		throw new UnsupportedOperationException("please using other construction function");
	}

	public PageBean(int pageno) {
		Assert.isTrue(pageno>0,"pageno must be greater than zero");
		this.pageno =pageno;
	}
	
	public PageBean(int pageno,int pagesize) {
		Assert.isTrue(pageno>0,"pageno must be greater than zero");
		Assert.isTrue(pagesize>0,"pagesize must be greater than zero");
		this.pageno=pageno;
		this.pagesize =pagesize;
	}
	
	public int getPageno() {
		return pageno;
	}
	
	public PageBean setPageno(int pageno){
		this.pageno=pageno;
		return this;
	}

	public PageBean setPagesize(int pagesize){
		this.pagesize=pagesize;
		return this;
	}
	
	public int getPagesize() {
		return pagesize;
	}

	public long getTotalPage() {
		if(totalCount%pagesize==0) {
			return totalCount/pagesize;
		}else{
			return (totalCount-totalCount%pagesize)/pagesize+1;
		}
	}
	
	public int getNextPage() {
		if (hasNextPage()) {
			return pageno + 1;
		}
		return pageno;
	}
	
	public int getPrevPage(){
		if(hasPrevPage()){
			return pageno-1;
		}
		return pageno;
	}

	public boolean hasPrevPage(){
		return pageno>1;
	}

	public boolean hasNextPage() {
		return pageno < totalPage;
	}

	
	public long getTotalCount(){
		return totalCount;
	}
	
	public void setTotalCount(long totalCount){
		this.totalCount=totalCount;
	}
	
	public int getStartIndex(){
		return pagesize*(pageno-1);
	}
	
	public int getEndIndex(){
		return pageno*pagesize;
	}
}
