/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jxva.dao.jdbc;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 *
 * @author  The Jxva Framework Foundation
 * @since   1.0
 * @version 2009-03-09 12:01:09 by Jxva
 */
public class CaseInsensitiveLinkedHashMap<K,V> extends LinkedHashMap<K,V> {
	
	private static final long serialVersionUID = 1L;
	
	public CaseInsensitiveLinkedHashMap(){
		super();
	}
	
	public CaseInsensitiveLinkedHashMap(int initialCapacity){
		super(initialCapacity);
	}


	public V get(Object key){
		 if (key == null)return null;
		 return super.get(key.toString().toLowerCase());
	}
	
	@SuppressWarnings("unchecked")
	public V put(K key,V value){
		if (key == null||value==null)return null;
		return super.put((K)key.toString().toLowerCase(), value);
	}
	
	public boolean containsKey(String key){
		if (key == null)return false;
		return super.containsKey(key.toLowerCase());
	}

	public V remove(Object key) {
		if(key==null)return null;
		return super.remove(key.toString().toLowerCase());
	}
	
	public static void main(String[] args){
		 System.out.println("===== testCaseInsensitiveMap =====");
		 Map<String,String> map = new CaseInsensitiveLinkedHashMap<String,String>();
	     map.put("key1", "value1");
	     map.put("key2", "value2");
	     map.put("KeY1", "value3");
	     map.put("Key4", "value4");
	     map.put("Key5", "value5");
	     
	     for(String s:map.keySet()){
	    	 System.out.println(s+"="+map.get(s));
	     }
	}

}
