/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jxva.mvc;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.jxva.dao.DAOHolder;
import com.jxva.mvc.entity.BaseServlet;
import com.jxva.mvc.entity.View;
import com.jxva.plugin.MvcPlugin;
import com.jxva.plugin.SysPlugin;

/**
 * framework core controller<br>
 * 根据用户请求调用相应的业务处理类,并根据执行结果或注解值导航<br>
 * 需要在web.xml中配置
 * <pre>
 * &lt;servlet&gt;
 *   &lt;servlet-name&gt;Controller&lt;/servlet-name&gt;
 *     &lt;servlet-class&gt;com.jxva.mvc.Controller&lt;/servlet-class&gt;
 *   &lt;/servlet&gt;
 *   &lt;servlet-mapping&gt;
 *     &lt;servlet-name>Controller&lt;/servlet-name&gt;
 *     &lt;url-pattern>*.jv&lt;/url-pattern&gt;
 *  &lt;/servlet-mapping&gt;
 * </pre>
 * @author  The Jxva Framework
 * @since   1.0
 * @version 2008-11-27 10:00:53 by Jxva
 */
public class Controller extends BaseServlet{
	
	private static final long serialVersionUID = 1L;
	
	public void service(HttpServletRequest request,HttpServletResponse response)throws ServletException{
		response.setCharacterEncoding("UTF-8");
		response.setContentType("text/html;charset=UTF-8");
//		response.setHeader("Pragma", "No-cache");
//		response.setHeader("Cache-Control", "no-cache");
//		response.setDateHeader("Expires", 0);
		try{
			final String[] actionMethod=Util.getActionAndMethod(request.getServletPath());
			final Object action=MvcPlugin.getAction(actionMethod[0]);
			if(action==null){
				int lastPos=actionMethod[0].lastIndexOf('/')+1;
				View.printInfoWithHeaderAndFooter(response.getWriter(),"There is no Action mapped for namespace "+actionMethod[0].substring(0,lastPos)+" and action name '"+actionMethod[0].substring(lastPos)+"'.");
				return;
			}
			final String result=Util.execute(request,response,action,actionMethod[1]);
			if(response.isCommitted()){
				return;
			}else{
				final Object[] objs=action.getClass().getDeclaredMethod(actionMethod[1]).getDeclaredAnnotations();
				if(objs.length>0){
					Object annotation=objs[0];
					if(annotation instanceof Forward){
						forward(request,response,Util.getValue(result,((Forward)annotation).value()));
					}else if(annotation instanceof Redirect){
						redirect(request,response,Util.getValue(result,((Redirect)annotation).value()));
					}else if(annotation instanceof Perform){
						return;
					}else{
						View.printInfo(response.getWriter(),result);
						return;
					}
				}else{
					if(result==null){
						View.printInfoWithHeaderAndFooter(response.getWriter(),action.getClass().getName()+'.'+actionMethod[1]+"() can't return null.");
						return;
					}
					View.printInfo(response.getWriter(),result);
				}
			}
		}catch(Exception e){
			try{
				e.printStackTrace();
				if(SysPlugin.getDebug()){
					View.printExceptionWithHeaderAndFooter(response.getWriter(),e);
				}else{
					forward(request,response,SysPlugin.getErrorPage());
				}
			}catch(IOException ex){
				ex.printStackTrace();
			}
		}
	}
		
	private final static class Util{
		
		public static  String[] getActionAndMethod(String url){
			String[] s=new String[2];
			if(url.indexOf('!')>-1){
				String[] actionArray=url.substring(0,url.lastIndexOf('.')).split("\\!");
				s[0]=actionArray[0];
				s[1]=actionArray[1];
			}else{
				s[0]=url.substring(0,url.lastIndexOf('.'));
				s[1]="execute";
			}
			return s;
		}
		
		public static String execute(HttpServletRequest request,HttpServletResponse response,Object obj,String ops)throws Exception{
			try{	
				if(obj instanceof Action){
					((Action) obj).init(request, response);
				}
				return (String)obj.getClass().getMethod(ops).invoke(obj);
			}catch(InvocationTargetException e){
				e.getTargetException().printStackTrace();
				View.printInvocationTargetExceptionWithHeaderAndFooter(response.getWriter(),e);
				return null;
			}finally{
				if(SysPlugin.getUseDao()){
					DAOHolder.removeDAO();
				}
			}
		}
		
		private static String getValue(final String result,final String value){
			if(result==null)return value;
			if(value.length()==0){
				return result;
			}else{
				return value.replaceAll("\\*",result);
			}
		}
	}
}
