/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jxva.mvc;

import java.io.IOException;

import javax.imageio.ImageIO;

import com.jxva.dao.DAOFactory;
import com.jxva.entity.RandomCode;
import com.jxva.entity.ValidCode;
import com.jxva.mvc.entity.View;
import com.jxva.plugin.Plugin;
import com.jxva.plugin.PluginException;

/**
 * 
 * Framework Visual Monitor and Manage
 * @author  The Jxva Framework
 * @since   1.0
 * @version 2008-11-27 10:17:53 by Jxva
 */
public class FrameworkAction extends Action{
	
	public String execute(){
		String urlExt=getUrlExt();
		StringBuilder sb=new StringBuilder();
		sb.append(" <a href='framework!dbPool").append(urlExt).append("'>Database Connection Pool</a> ");
		sb.append(" <a href='framework!validCode").append(urlExt).append("'>Display ValidCode</a> ");
		sb.append(" <a href='framework!reInitialize").append(urlExt).append("'>ReInitialize Configuration</a> ");
		return View.getInfoWithHeaderAndFooter(sb.toString());
	}
	
	public String dbPool(){
		return View.getInfoWithHeaderAndFooter(DAOFactory.getInstance().getConnectionProvider().getStatistics().toString());
	}
		
	public String reInitialize() throws PluginException{
		new Plugin().reInitialize();
		return "reInitialize success";
	}
	
	@Perform
	public void validCodeNumber() throws IOException{
		response.setHeader("Pragma","No-cache");  
		response.setHeader("Cache-Control","no-cache");  
		response.setDateHeader("Expires", 0);  
		response.reset();  
		response.setContentType("image/jpeg");
		RandomCode randomCode=new RandomCode();
		ImageIO.write(randomCode.createNumber(),"JPEG",response.getOutputStream());  
		session.setAttribute("randomCode",randomCode.getFlagInSession());
	}
	
	@Perform
	public void validCodeAlpha() throws IOException{
		response.setHeader("Pragma","No-cache");  
		response.setHeader("Cache-Control","no-cache");  
		response.setDateHeader("Expires", 0);  
		response.reset();  
		response.setContentType("image/jpeg");
		RandomCode randomCode=new RandomCode();
		ImageIO.write(randomCode.createAlpha(),"JPEG",response.getOutputStream());  
		session.setAttribute("randomCode",randomCode.getFlagInSession());
	}
	
	@Perform
	public void validCodeChinese() throws IOException{
		response.setHeader("Pragma","No-cache");  
		response.setHeader("Cache-Control","no-cache");  
		response.setDateHeader("Expires", 0);  
		response.reset();  
		response.setContentType("image/jpeg");
		RandomCode randomCode=new RandomCode();
		ImageIO.write(randomCode.createChinese(),"JPEG",response.getOutputStream());  
		session.setAttribute("randomCode",randomCode.getFlagInSession());
	}
	
	@Perform
	public void validCode(){
		ValidCode.create(request, response,"validCode");
	}
	
	private String getUrlExt(){
		String url=request.getRequestURL().toString();
		return url.substring(url.lastIndexOf('.'));
	}
}
