/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.jxva.tag;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * @author  The Jxva Framework Foundation
 * @since 2006-12-16 <j:option value="" selected="true|false"/>text</j:option>
 */

public class OptionTag extends BaseInputTag {

	private static final long serialVersionUID = 1L;

	protected String bc;

	public int doStartTag() throws JspException {
		this.bc=null;
		return EVAL_BODY_AGAIN;
	}

	public int doAfterBody() throws JspException{
		if (bodyContent != null) {
            String text = bodyContent.getString();

            if (text != null) {
                text = text.trim();

                if (text.length() > 0) {
                    this.bc = text;
                }
            }
        }
		return SKIP_BODY;
	}
	
	public int doEndTag() throws JspException {
		try {
						
			SelectTag selecttag = (SelectTag)TagSupport.findAncestorWithClass(this,SelectTag.class);
			if(selecttag!=null){
				bean = selecttag.getName();
			}
			
			StringBuilder sb = new StringBuilder(128);
			sb.append("<option value=");
			sb.append('"').append(value).append('"');

			if (disabled) {
				sb.append(" disabled=\"true\"");
			}
			
			if (pageContext.getAttribute(bean) != null) {
				result =TagUtil.formatValue(pageContext.getAttribute(bean));
			}
			Object defaultValue=pageContext.getAttribute(bean+".value");

			if (!hasBeanValue()) {
				if (selected)sb.append(" selected=\"true\"");
			} else if(defaultValue!=null) {
				if(value.equals(TagUtil.formatValue(defaultValue))){
					sb.append(" selected=\"true\"");
				}
			}else{
				if(result.indexOf(',')>-1){
					String[] values=result.split("\\,");
					for(int k=0;k<values.length;k++){
						if(values[k].equals(value)){
							sb.append(" selected=\"true\"");
							break;
						}
					}
				}else{
					if(value.equals(result)){
						sb.append(" selected=\"true\"");
					}
				}
			}

			sb.append('>');
			sb.append(bc==null?value:bc);
			sb.append("</option>");
			pageContext.getOut().print(sb.toString());
			return EVAL_PAGE;
		//}catch(NullPointerException e){
			
		}catch (IOException e) {
			throw new JspException(e);
		}

	}


}