
/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.jxva.tag;

import java.io.IOException;
import java.util.regex.Pattern;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

import com.jxva.util.HtmlUtil;
import com.jxva.util.StringUtil;

/**
 * 
   <j:property value="var" filter="false" length=""/>
   <tag>
    <name>property</name>
    <tag-class>com.jxva.tag.PropertyTag</tag-class>
    <body-content>empty</body-content>
    <attribute>
        <name>value</name>
        <required>true</required>
        <rtexprvalue>false</rtexprvalue>
    </attribute>
    <attribute>
        <name>filter</name>
        <required>false</required>
        <rtexprvalue>true</rtexprvalue>
    </attribute>
    <attribute>
        <name>clear</name>
        <required>false</required>
        <rtexprvalue>true</rtexprvalue>
    </attribute>
    <attribute>
        <name>length</name>
        <required>false</required>
        <rtexprvalue>true</rtexprvalue>
    </attribute>
  </tag>
  
 *
 * @author  The Jxva Framework Foundation
 * @since   1.0
 * @version 2008-09-05 23:53:44 by Jxva
 */
public class PropertyTag extends TagSupport {

	private static final long serialVersionUID = 1L;

	protected String value;
	protected boolean filter;
	protected boolean clear;
	protected int length;
	
	public void setValue(String value) {
		this.value = value;
	}
	
	public void setFilter(boolean filter){
		this.filter=filter;
	}
	
	public void setLength(int length) {
		this.length = length;
	}
	
	public void setClear(boolean clear) {
		this.clear = clear;
	}

	public int doStartTag() throws JspException {
		String result=null;
		Tag tag =  TagSupport.findAncestorWithClass(this,IterateTag.class);
		if(tag!=null){
			String iterateName=((IterateTag)tag).getName();
			if(Pattern.compile(iterateName+"\\[\\d+\\]").matcher(value).find()){//for array
				result=TagUtil.findValue(pageContext,iterateName+'.'+value);
			}else{
				if(value.indexOf(IterateTag.INDEX_FLAG)>-1){
					result=TagUtil.findValue(pageContext,value);
				}else{
					if(value.indexOf(iterateName+'.')>-1){
						result=TagUtil.findValue(pageContext,value);
					}else{
						result=TagUtil.findValue(pageContext,iterateName+'.'+value);
					}
				}
			}
			if(result==null){
				result=TagUtil.findValue(pageContext,value);
			}
		}else{
			result=TagUtil.findValue(pageContext,value);
		}
		if(filter){
			result=HtmlUtil.filter(result);
		}
		if(clear){
			result=HtmlUtil.clear(result);
		}
		if(length>0){
			result=StringUtil.getUnicodeSubString(result,0,length);
		}
		try {
			pageContext.getOut().print(result);
			return EVAL_BODY_INCLUDE;
		} catch (IOException e) {
			throw new JspException(e);
		}
	}

	public int doEndTag() throws JspException {
		return EVAL_PAGE;
	}
}
