
/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.jxva.tag;

import javax.el.ELContext;
import javax.el.ExpressionFactory;
import javax.el.ValueExpression;
import javax.servlet.jsp.JspFactory;
import javax.servlet.jsp.PageContext;

/**
 * 
 * @author  The Jxva Framework Foundation
 * @since 2006-12-16
 */
public abstract class TagUtil {


	/**
	 * <b>增加属性</b> <br>
	 * 例如:<br>
	 * renderAttribute(sb, "value", getValue());
	 * 
	 * @param results
	 *            缓冲的字符串
	 * @param attribute
	 *            属性名
	 * @param value
	 *            属性值
	 */
	public static void addAttribute(StringBuilder sb, String attribute,String value) {
		if (value != null) {
			sb.append(' ');
			sb.append(attribute);
			sb.append("=\"");
			sb.append(value);
			sb.append('"');
		}
	}

	/**
	 * 通过名称得到Scope中的对象 执行顺序是page->request->session->application
	 * 
	 * @param pageContext
	 * @param name
	 * @return Object
	 */
	public static Object getScopeObject(PageContext pageContext, String name) {
		if (pageContext.getAttribute(name, PageContext.PAGE_SCOPE) != null) {
			return pageContext.getAttribute(name, PageContext.PAGE_SCOPE);
		} else if (pageContext.getAttribute(name, PageContext.REQUEST_SCOPE) != null) {
			return pageContext.getAttribute(name, PageContext.REQUEST_SCOPE);
		} else if (pageContext.getAttribute(name, PageContext.SESSION_SCOPE) != null) {
			return pageContext.getAttribute(name, PageContext.SESSION_SCOPE);
		} else if (pageContext.getAttribute(name, PageContext.APPLICATION_SCOPE) != null) {
			return pageContext.getAttribute(name, PageContext.APPLICATION_SCOPE);
		} else {
			return null;
		}
	}

	public static String formatValue(Object obj) {
		if (obj instanceof java.lang.String) {
			return (String) obj;
		} else if (obj instanceof Number) {
			return obj + "";
		} else if (obj instanceof java.util.Date) {
			return obj + "";
		} else if (obj instanceof java.lang.Character) {
			return obj + "";
		} else {
			return null;
		}
	}
	
	public static Object findObject(PageContext pageContext,String expression){
		return findValue(pageContext,expression,Object.class);
	}
	
	public static Object findValue(PageContext pageContext,String expression,Class<?> type){
		try{
			ELContext elContext=pageContext.getELContext();
			ExpressionFactory expressionFactory =JspFactory.getDefaultFactory().getJspApplicationContext(pageContext.getServletContext()).getExpressionFactory();     
			ValueExpression ve = expressionFactory.createValueExpression(elContext,"#{"+expression+"}",type);
			return ve.getValue(elContext);
		}catch(Exception e){
			e.printStackTrace();
			return null;
		}
	}
	
	public static String findValue(PageContext pageContext,String expression){
		return formatValue(findValue(pageContext,expression,String.class));
	}
}