/*
 * Copyright @ 2006-2009 by The Jxva Framework Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jxva.cache;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.jxva.cache.Cache;
import com.jxva.cache.soft.FIFOSoftCache;
import com.jxva.cache.soft.FastSoftCache;
import com.jxva.cache.soft.LFUSoftCache;
import com.jxva.cache.soft.LRUSoftCache;
import com.jxva.cache.soft.MRUSoftCache;


/**
  singleThread:100
  		lru:  18810491
		fifo: 14127773
		lfu:  27219661
		mru:  7704331
		fast: 19383190
  singleThread:1000
	  	lru:  59320389
		fifo: 21210797
		lfu:  36459941
		mru:  22961019
		fast: 44828882
  singleThread:10000
		lru:  174915806
		fifo: 141299802
		lfu:  210197385
		mru:  162861201
		fast: 271664415
  singleThread:10000
		lru:  13188227
		fifo: 7244979
		lfu:  16299725
		mru:  6537091
		fast: 12688371
 * @author  The Jxva Framework Foundation
 * @since   1.0
 * @version 2009-01-23 23:30:15 by Jxva
 */
public class CacheTest {

	public static void main(String[] args) {
        singleThread(); 
		//multiThread();
	}
	
	public static void multiThread(){
		ExecutorService exec = Executors.newCachedThreadPool();
		for (int index = 0; index < 10000; index++) {
			Runnable run = new Runnable() {
				public void run() {
					demo(new LRUSoftCache("lru",5));
					demo(new FIFOSoftCache("fifo",5));
					demo(new LFUSoftCache("lfu",5));
					demo(new MRUSoftCache("mru",5));
					demo(new FastSoftCache("fast",5));
				}
			};
			exec.execute(run);
		}
		exec.shutdown();
	}
	
	public static void singleThread(){
        System.out.println("lru:  "+test(new LRUSoftCache("lru",5))); 
        System.out.println("fifo: "+test(new FIFOSoftCache("fifo",5))); 
        System.out.println("lfu:  "+test(new LFUSoftCache("lfu",5))); 
        System.out.println("mru:  "+test(new MRUSoftCache("mru",5))); 
        System.out.println("fast: "+test(new FastSoftCache("fast",5))); 
	}
	
	public static long test(Cache cache){
		long s=System.nanoTime();
		for(int i=0;i<100;i++){
			demo(cache);
		}
		return System.nanoTime()-s;
	}
	
	public static void demo(Cache cache){
		logln(cache.getName()+"--------------------------------");
        cache.put(1, "1");  
        cache.put(2, "2");  
        cache.put(3, "3");  
        cache.put(4, "4");  
        cache.getValue(4);
        cache.put(5, "5");  
        cache.put(6, "6");  
        cache.put(7, "7");  
        cache.getValue(4);
        cache.put(8, "8");  
        cache.put(9, "9");  
 
        log(cache.getValue(4));   
        
        Object[] keys= cache.getKeys(); 
        logln(keys.length);
        for (Object key:keys) {  
        	log(cache.getValue(key));
        	log(cache.containsKey(key)); 
        	log(cache.getKeys().length); 
        	log(cache.getQuiet(key)); 
        	log(cache.remove(key)); 
        	logln(cache.get(key));  
        } 		
	}
	
	public static void log(Object obj){
		//System.out.print(obj);
	}
	
	public static void logln(Object obj){
		//System.out.println(obj);
	}
}
